@@ -176,17 +176,15 @@ macro_rules! impl_object_methods {
         let mut error_code = ptr::null_mut();
         let mut reason_string = ptr::null_mut();
         let mut js_error = ptr::null_mut();
-        let create_code_status = sys::napi_create_string_utf8(
-          env,
-          error_code_string.as_ptr(),
-          status_len,
-          &mut error_code,
-        );
+        let create_code_status = unsafe {
+          sys::napi_create_string_utf8(env, error_code_string.as_ptr(), status_len, &mut error_code)
+        };
         debug_assert!(create_code_status == sys::Status::napi_ok);
-        let create_reason_status =
-          sys::napi_create_string_utf8(env, reason.as_ptr(), reason_len, &mut reason_string);
+        let create_reason_status = unsafe {
+          sys::napi_create_string_utf8(env, reason.as_ptr(), reason_len, &mut reason_string)
+        };
         debug_assert!(create_reason_status == sys::Status::napi_ok);
-        let create_error_status = $kind(env, error_code, reason_string, &mut js_error);
+        let create_error_status = unsafe { $kind(env, error_code, reason_string, &mut js_error) };
         debug_assert!(create_error_status == sys::Status::napi_ok);
         js_error
       }
@@ -201,10 +199,10 @@ macro_rules! impl_object_methods {
         if status == Status::PendingException {
           return;
         }
-        let js_error = self.into_value(env);
+        let js_error = unsafe { self.into_value(env) };
         #[cfg(debug_assertions)]
-        let throw_status = sys::napi_throw(env, js_error);
-        sys::napi_throw(env, js_error);
+        let throw_status = unsafe { sys::napi_throw(env, js_error) };
+        unsafe { sys::napi_throw(env, js_error) };
         #[cfg(debug_assertions)]
         assert!(
           throw_status == sys::Status::napi_ok,
